# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['selic']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2', 'requests>=2.25.0,<3.0.0']

setup_kwargs = {
    'name': 'selic',
    'version': '0.1.0b0',
    'description': 'Fetch SELIC data',
    'long_description': None,
    'author': 'tilacog',
    'author_email': 'tilacog@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/tilacog/selic',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import csv
import sys
from dataclasses import astuple, dataclass
from datetime import date, datetime
from decimal import Decimal
from itertools import starmap, zip_longest

import bs4
import requests

URL = "http://sal.receita.fazenda.gov.br/PortalSalInternet/faces/pages/tabelaPratica/exibirTabela.xhtml"  # noqa


@dataclass
class Selic:
    period: date
    ufir: Decimal
    interest: Decimal

    @classmethod
    def from_triple(cls, period, ufir, interest):
        numbers = map(Decimal, map(lambda x: 0 if x == "-" else x, (ufir, interest)))
        date_ = datetime.strptime(period, "%m/%Y").date()
        return cls(date_, *numbers)


def grouper(iterable, n, fillvalue=None):
    "Collect data into fixed-length chunks or blocks"
    # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * n
    return zip_longest(*args, fillvalue=fillvalue)


def extract(url):
    response = requests.get(url)
    soup = bs4.BeautifulSoup(response.text, "html.parser")

    rows = (
        soup.find(id="formExibirTabela:gridListSalariosCalculo")
        .find("tbody")
        .find_all("tr")
    )
    for ridx, row in enumerate(rows):
        cells = row.find_all("span")
        triples = starmap(Selic.from_triple, grouper((cell.text for cell in cells), 3))
        yield from triples


def main():
    writer = csv.writer(sys.stdout)
    writer.writerow("Competência UFIR Juros".split())
    iterator = sorted(map(astuple, extract(URL)), key=lambda x: x[0])
    writer.writerows(iterator)


if __name__ == "__main__":
    main()
